with System.Storage_Elements;

package body Counter
  with SPARK_Mode

-- Small external state tutorial exercise
--
-- This package uses an external counter which is incremented
-- by one every time:
-- A. its clock ticks,
-- B. it is read.
--
-- It has one query procedure Bump_And_Monitor to check if the
-- counter has reached its limit.

is
   -- 1. Add aspects to the declaration of the external
   --    state variable Port, to indicate:
   --      - that it is volatile
   --      - what sort of inputs/outputs it has.
   Port : Integer
     with Volatile, Async_Writers, Effective_Reads,
          -- address clause would be added here, for example:
          Address => System.Storage_Elements.To_Address (16#ACECAF0#);

   Limit : constant Integer := 3_000_000;

   -- 2. Add Global and Depends contracts to the subprogram
   --    declarations in the package specification.
   --    Use the SPARK 2014 tools to check your contracts.

   procedure Bump_And_Monitor (Alarm : out Boolean)
     with
       Global  => (In_Out => Port),
       Depends => (Alarm => Port,
                   Port  => Port)
   is
      Calc : Integer;
   begin
      Calc := Port;
      Alarm := Limit < Calc;
   end Bump_And_Monitor;

end Counter;
