package Bounded_Queue with SPARK_Mode is

   -- non-encapsulated version with contracts for successfull AoRTE

   subtype Element_Type is Integer;

   type Queue_Array is array (Positive range <>) of Element_Type;
   type Queue_Type (Max_Size : Positive) is
      record
        Count : Natural:= 0;    -- Number of items
        Front : Positive := 1;   -- Index of first item
        Rear  : Positive := 1;   -- Index of last item
        Items : Queue_Array (1 .. Max_Size);  -- The element array
      end record
   with
     Dynamic_Predicate => Front in 1 .. Max_Size and
                          Rear in 1 .. Max_Size and
                          Count <= Max_Size;

   function Full (Queue : in Queue_Type) return Boolean
     is (Queue.Count = Queue.Max_Size);

   function Empty (Queue : in Queue_Type) return Boolean
     is (Queue.Count = 0);

   function Size (Queue : in Queue_Type) return Natural;

   function First_Element (Queue : in Queue_Type) return Element_Type
   with
     Pre => not Empty (Queue);

   function Last_Element (Queue : in Queue_Type) return Element_Type
   with
         Pre => not Empty (Queue);

   procedure Clear (Queue : in out Queue_Type)
      with
       Post => Empty (Queue);

   procedure Enqueue (Queue : in out Queue_Type;
                      Item  : in     Element_Type)
      with
         Pre  => not Full (Queue);

   procedure Dequeue (Queue : in out Queue_Type;
                      Item  :    out Element_Type)
      with
         Pre  => not Empty (Queue);

end Bounded_Queue;
