generic

   type Element_Type is private;

package Bounded_Queue with SPARK_Mode is

   -- generic encapsulated version of the Queue

   type Queue_Type (Max_Size : Positive) is private;

   function Size (Queue : in Queue_Type) return Natural;

   function Full (Queue : in Queue_Type) return Boolean
     is (Size (Queue) = Queue.Max_Size);

   function Empty (Queue : in Queue_Type) return Boolean
     is (Size (Queue) = 0);

   function First_Element (Queue : in Queue_Type) return Element_Type
   with
     Pre => not Empty (Queue);

   function Last_Element (Queue : in Queue_Type) return Element_Type
   with
         Pre => not Empty (Queue);

   procedure Clear (Queue : in out Queue_Type)
      with
       Post => Empty (Queue);

   procedure Enqueue (Queue : in out Queue_Type;
                      Item  : in     Element_Type)
      with
         Pre  => not Full (Queue);

   procedure Dequeue (Queue : in out Queue_Type;
                      Item  :    out Element_Type)
      with
       Pre  => not Empty (Queue);

private

   type Queue_Array is array (Positive range <>) of Element_Type;
   type Queue_Type (Max_Size : Positive) is
      record
        Count : Natural:= 0;    -- Number of items
        Front : Positive := 1;   -- Index of first item
        Rear  : Positive := 1;   -- Index of last item
        Items : Queue_Array (1 .. Max_Size);  -- The element array
      end record
   with
       Dynamic_Predicate => Front in 1 .. Max_Size and
                            Rear in 1 .. Max_Size and
                            Count <= Max_Size;

   function Size (Queue : in Queue_Type) return Natural
     is (Queue.Count);

end Bounded_Queue;
