with Ada.Text_IO;   use Ada.Text_IO;
with Display;       use Display;
with Display.Basic; use Display.Basic;

procedure Main is

   --  Logical coordinate system
   X_Size : constant Natural := 8;
   Y_Size : constant Natural := 16;  
   subtype X_Coord is Natural range 1 .. X_Size;
   subtype Y_Coord is Natural range 1 .. Y_Size;
     
   type Cell_Type is (Empty, I, O, J, L, S, T, Z);
   
   type Row   is array (X_Coord) of Cell_Type;
   type Board is array (Y_Coord) of Row;
   
   --  Pixel constants for graphical components
   Pxl_Width    : constant Natural := 10;
   Pxl_Origin_X : constant Integer := -40;
   Pxl_Origin_Y : constant Integer := 80;
   
   Game_Board : Board;
   Falling_Piece : Shape_Id;
   
begin
   
   Game_Board := (others => (others => Empty));
   
   Falling_Piece := 
     New_Box (X      => Float (Pxl_Origin_X + X_Size / 2 * Pxl_Width),
              Y      => Float (Pxl_Origin_Y),
              Width  => Float (Pxl_Width),
              Height => Float (Pxl_Width),
              Color  => Red);
   
   for Y in Y_Coord loop
      delay 0.2;
      Set_Y (Falling_Piece, Float (Pxl_Origin_Y - Y * Pxl_Width));
   end loop;
   
   --  Integration into the board
   Game_Board (Y_Size) (X_Size / 2) := Z;
   
   Put_Line ("GAME OVER!");
   
   --  print board
   for Y in Y_Coord loop
      for X in X_Coord loop
         Put (if Game_Board (Y) (X) = Empty then "- " else 
                 Cell_Type'Image (Game_Board (Y) (X)) & " ");   
      end loop;
      Put_Line (" ");
   end loop;
   Put_Line (" ");

end Main;
