with Ada.Text_IO;   use Ada.Text_IO;
with Display;       use Display;
with Display.Basic; use Display.Basic;

procedure Main is

   --  Logical coordinate system
   X_Size : constant Natural := 8;
   Y_Size : constant Natural := 16;  
   subtype X_Coord is Natural range 1 .. X_Size;
   subtype Y_Coord is Natural range 1 .. Y_Size;
   
   type Cell_Type is (Empty, I, O, J, L, S, T, Z);
   -- I: cyan, J: blue, L: white, O: yellow, S: green, T: magenta, Z: red
   
   subtype Piece_Type is Cell_Type range I .. Z;
   
   type Cell is record
      Kind     : Cell_Type;
      Graphics : Shape_Id;
   end record;
   
   type Piece is record
      Kind     : Piece_Type;
      Y        : Y_Coord;
      X        : X_Coord;
      Graphics : Shape_Id;
   end record;
   
   type Row   is array (X_Coord) of Cell;
   type Board is array (Y_Coord) of Row;
   
   --  Pixel constants for graphical components
   Pxl_Width    : constant Natural := 10;
   Pxl_Origin_X : constant Integer := -40;
   Pxl_Origin_Y : constant Integer := 80;
   
   Game_Board : Board;
   
   Falling_Piece : Piece;
   
begin
   Game_Board := (others => (others => Cell' (Kind     => Empty, 
                                              Graphics => Null_Shape_Id)));
      
   Falling_Piece := 
     (Kind     => Z,
      X        => X_Size / 2,
      Y        => 1,
      Graphics => 
        New_Box (X      => Float (Pxl_Origin_X + X_Size / 2 * Pxl_Width),
                 Y      => Float (Pxl_Origin_Y),
                 Width  => Float (Pxl_Width),
                 Height => Float (Pxl_Width),
                 Color  => Red));
   
   for Y in Y_Coord loop
      delay 0.2;
      Falling_Piece.Y := Y;
      Set_Y (Falling_Piece.Graphics, 
             Float (Pxl_Origin_Y - Falling_Piece.Y * Pxl_Width));
   end loop;
   
   Game_Board (Falling_Piece.Y) (Falling_Piece.X) := 
     (Kind     => Falling_Piece.Kind, 
      Graphics => Falling_Piece.Graphics);
   
   Put_Line ("GAME OVER!");
   
      --  print board
   for Y in Y_Coord loop
      for X in X_Coord loop
         Put (if Game_Board (Y) (X).Kind = Empty then "- " else 
                 Cell_Type'Image (Game_Board (Y) (X).Kind) & " ");   
      end loop;
      Put_Line (" ");
   end loop;
   Put_Line (" ");

end Main;
