package body Tetris
with SPARK_Mode => On
is
   --  Pixel constants for graphical components
   Pxl_Width    : constant Natural := 10;
   Pxl_Origin_X : constant Integer := -40;
   Pxl_Origin_Y : constant Integer := 80;

   function New_Board return Board
   is
   begin
      return Board' (others => (others => Cell' (Kind     => Empty,
                                                 Graphics => Null_Shape_Id)));
   end New_Board;

   function New_Piece (Kind : Piece_Type) return Piece
   is
      Color   : Color_Type;
      Start_Y : constant Y_Coord := 1;
      Start_X : constant X_Coord := X_Size / 2;
   begin
      Color := (case Kind is
                   when I => Cyan,
                   when J => Blue,
                   when L => White,
                   when O => Yellow,
                   when S => Green,
                   when T => Magenta,
                   when Z => Red);

      return (Kind     => Kind,
              X        => Start_X,
              Y        => Start_Y,
              Graphics =>
                 New_Box (X      => Float (Pxl_Origin_X + Start_X * Pxl_Width),
                          Y      => Float (Pxl_Origin_Y - Start_Y * Pxl_Width),
                          Width  => Float (Pxl_Width),
                          Height => Float (Pxl_Width),
                          Color  => Color));
   end New_Piece;

   procedure Do_Action (Desired_Move : in     Action;
                        Cur_Piece    : in out Piece)
   is
   begin
      if Move_Is_Within_Board (Cur_Piece, Desired_Move) then
         Cur_Piece := Move (Cur_Piece, Desired_Move);
         return;
      end if;
   end Do_Action;

   procedure Insert_Piece_In_Board (P: in Piece; B: in out Board) is
   begin
      B (P.Y) (P.X) := (Kind     => P.Kind,
                        Graphics => P.Graphics);
   end Insert_Piece_In_Board;

   procedure Update_Graphics (P: in out Piece) is
   begin
      Set_X (P.Graphics, Float (Pxl_Origin_X + P.X * Pxl_Width));
      Set_Y (P.Graphics, Float (Pxl_Origin_Y - P.Y * Pxl_Width));
   end Update_Graphics;

end Tetris;
