with Display;       use Display;
with Display.Basic; use Display.Basic;

package Tetris
with SPARK_Mode => On
is

   --  Logical coordinate system
   X_Size : constant Natural := 8;
   Y_Size : constant Natural := 16;
   subtype X_Coord is Natural range 1 .. X_Size;
   subtype Y_Coord is Natural range 1 .. Y_Size;

   type Cell_Type is (Empty, I, O, J, L, S, T, Z);
   -- I: cyan, J: blue, L: white, O: yellow, S: green, T: magenta, Z: red

   subtype Piece_Type is Cell_Type range I .. Z;

   type Cell is record
      Kind     : Cell_Type;
      Graphics : Shape_Id;
   end record;

   type Piece is record
      Kind     : Piece_Type;
      Y        : Y_Coord;
      X        : X_Coord;
      Graphics : Shape_Id;
   end record;

   type Row   is array (X_Coord) of Cell;
   type Board is array (Y_Coord) of Row;

   type Action is (Move_Left, Move_Right, Move_None, Move_Down);
   subtype User_Action is Action range Move_Left .. Move_None;

   function New_Board return Board;

   function New_Piece (Kind : Piece_Type) return Piece;

   function Move_Is_Within_Board (P : Piece; A : Action) return Boolean is
      (case A is
          when Move_Left   => P.X - 1 in X_Coord,
          when Move_Right  => P.X + 1 in X_Coord,
          when Move_None   => True,
          when Move_Down   => P.Y + 1 in Y_Coord);

   function Move (P : Piece; A : Action) return Piece is
     (case A is
         when Move_Left   => P'Update (X => P.X - 1),
         when Move_Right  => P'Update (X => P.X + 1),
         when Move_None  =>  P,
         when Move_Down   => P'Update (Y => P.Y + 1))
   with Pre => Move_Is_Within_Board (P, A);

   procedure Do_Action (Desired_Move : in     Action;
                        Cur_Piece    : in out Piece);

   procedure Insert_Piece_In_Board (P: in Piece; B: in out Board);

   procedure Update_Graphics (P: in out Piece);

end Tetris;
