with Ada.Text_IO;   use Ada.Text_IO;
with Tetris;        use Tetris;
with Display;       use Display;
with Display.Basic; use Display.Basic;
with Ada.Numerics.Discrete_Random;

procedure Main 
is
   
   package Random_Piece is 
     new Ada.Numerics.Discrete_Random (Piece_Type);
   use Random_Piece;
   
   Piece_Generator : Random_Piece.Generator;
   Next_Piece_Type : Piece_Type;
   
   Next_Move : User_Action;
   
   Game_Board : Board;
   
   Falling_Piece : Piece;
   
   OK, Found : Boolean;
   
   X : X_Coord;
   
   Last_Key : Key_Type;
   
begin
   
   Game_Board := New_Board;
   
   while not Game_Over (Game_Board)
     loop
      Next_Piece_Type := Random (Piece_Generator);
      Falling_Piece := New_Piece (Next_Piece_Type);
      OK := True;
      
      for Y in Y_Coord loop
         Last_Key := Current_Key_Press;
         Next_Move := (case To_Special (Last_Key) is
                          when KEY_LEFT => Move_Left,
                          when KEY_RIGHT => Move_Right,
                          when others => Move_None);
         
         Do_Action (Next_Move, Falling_Piece, Game_Board, OK);
         Do_Action (Move_Down, Falling_Piece, Game_Board, OK);
         delay 0.1;
         Update_Graphics (Falling_Piece);
         if not OK then
            exit;
         end if;
      end loop;
      Insert_Piece_In_Board (Cur_Piece => Falling_Piece,
                             Cur_Board => Game_Board);
      Delete_Complete_Line (Game_Board);
      
      --  print board
      for Y in Y_Coord loop
         for X in X_Coord loop
            Put ((if Game_Board (Y) (X).Kind = Empty then "-" else 
                    Cell_Type'Image (Game_Board (Y) (X).Kind)) & " ");   
         end loop;
         Put_Line (" ");
      end loop;
      Put_Line (" ");
      
      
   end loop;
   Put_Line ("GAME OVER!");
   
   -- query and print using fully specified search procedure
   Row_Has_Piece_Type (Z, Y_Coord'Last, Game_Board, Found, X);
   Put_Line ("Piece Z " & 
             (if Found then "found in column" & X_Coord'Image (X) 
                else "not found") &
               " on the bottom row."
            );
   
end Main;
