package body Cached_I_Sqrt.Cache_Ops.Cache with
   SPARK_Mode,
   Refined_State => (Basic_Cache => Last_16)
is

   type Entry_Pair is record
      Value : Natural;
      Result : Natural;
   end record;

   type Count_Type is range 0 .. 16;
   subtype Index is Count_Type range 1 .. Count_Type'Last;

   type Results_Array is array (Index) of Entry_Pair;

   type Last_16_Type is record
      Count : Count_Type;
      Posn  : Index;
      Results : Results_Array;
   end record;

   Last_16 : Last_16_Type;


   procedure Clear_Cache with
     SPARK_Mode => Off
   is
   begin
      Last_16.Count := Count_Type'Last;
      Last_16.Posn  := Index'First;
   end Clear_Cache;

   procedure Add (Value, Result : Natural) with
     Refined_Global => (In_Out => Last_16)
   is
   begin
      Last_16.Results (Last_16.Posn) := (Value => Value, Result => Result);
      if Last_16.Count < Count_Type'Last then
         Last_16.Count := Last_16.Count + 1;
      end if;
      if Last_16.Posn < Index'Last then
         Last_16.Posn := Last_16.Posn + 1;
      else
         Last_16.Posn := Index'First;
      end if;
   end Add;

   function Look_Up (N : Natural) return Look_Up_Result with
     Refined_Global => Last_16
   is
      Result : Natural := 0;
      Found : Boolean := False;
   begin
      for I in 1 .. Last_16.Count loop
         if Last_16.Results (I).Value = N then
            Result := Last_16.Results (I).Result;
            Found := True;
            exit;
         end if;
      end loop;
      if Found then
         return (Success => True, Result => Result);
      else
         return (Success => False);
      end if;
   end Look_Up;
begin
   Clear_Cache;
end Cached_I_Sqrt.Cache_Ops.Cache;
