with Cached_I_Sqrt.Cache_Ops;
package body Cached_I_Sqrt with
SPARK_Mode,
  Refined_State => (The_Cache => Cache_Ops.Hidden_Cache)
is
   function Non_Cached_I_Sqrt (N : Natural) return Natural with
     Post => Non_Cached_I_Sqrt'Result * Non_Cached_I_Sqrt'Result <= N and
       (Long_Long_Integer (Non_Cached_I_Sqrt'Result) + 1) *
       (Long_Long_Integer (Non_Cached_I_Sqrt'Result) + 1) > Long_Long_Integer (N)
   is
      Local_Root : Long_Long_Integer range 0..Long_Long_Integer'Last;
   begin
      Local_Root := 0;

      loop
         exit when (Local_Root + 1) * (Local_Root + 1) > Long_Long_Integer (N);
         pragma Loop_Invariant
           ((Local_Root + 1) * (Local_Root + 1) <= Long_Long_Integer (N));
         Local_Root := Local_Root + 1;
      end loop;
      return Natural (Local_Root);
   end Non_Cached_I_Sqrt;

   procedure I_Sqrt_Cached (N : Natural; Root : out Natural) with
     Global => (In_Out => Cache_Ops.Hidden_Cache)
   is
      Result : Look_Up_Result;
   begin
      Result := Cache_Ops.Look_Up (N);
      if Result.Success then
         Root := Result.Result;
      else
         Root := Non_Cached_I_Sqrt (N);
         Cache_Ops.Add (Value  => N,
                       Result => Root);
      end if;
   end I_Sqrt_Cached;

   function I_Sqrt (N : Natural) return Natural with
     SPARK_Mode => Off
   is
      Root : Natural;
   begin
      I_Sqrt_Cached (N, Root);
      return Root;
   end I_Sqrt;

end Cached_I_Sqrt;
