with Ada.Containers.Vectors;
package My_Vectors
  with SPARK_Mode
is
   subtype Index_Type is Positive;
   subtype Extended_Index is Index_Type'Base
      range Index_Type'First - 1 ..
            Index_Type'Min (Index_Type'Base'Last - 1, Index_Type'Last) + 1;
   subtype Element_Type is Integer;

   type Vector is private;

   Empty_Vector : constant Vector;

   function "&" (Left : Vector; Right : Element_Type) return Vector;

   function First_Index (V : Vector) return Index_Type;
   function Last_Index  (V : Vector) return Extended_Index;
   function Element (V : Vector; I : Index_Type) return Element_Type with
     Pre => I in First_Index (V) .. Last_Index (V);
   procedure Replace_Element (V  : in out Vector;
                              I  : Index_Type;
                              It : Element_Type) with
     Pre => I in First_Index (V) .. Last_Index (V);
private
   pragma SPARK_Mode (Off);

   package Vectors is new Ada.Containers.Vectors (Index_Type   => Index_Type,
                                                  Element_Type => Element_Type,
                                                  "="          => "=");
   type Vector is record
      Vec : Vectors.Vector;
   end record;

   Empty_Vector : constant Vector := (Vec => Vectors.Empty_Vector);
end My_Vectors;
