package Dictionaries
  with SPARK_Mode
is
   type Dictionary is private;
   type Element is private;
   type Key_Index is range 1 .. 64;

   type Key_String is array  (Key_Index range <>) of Character;

   function Count (Dic : Dictionary) return Natural;
   function Capacity (Dic : Dictionary) return Natural;
   function Has (Dic : Dictionary; X : Element) return Boolean;
   function Item (Dic : Dictionary; Key : Key_String) return Element with
     Pre => Key'Length > 0 and Key'First = 1;

   procedure Put (Dic : in out Dictionary; X : Element; Key : Key_String) with
   -- Insert X so that it will be retrievable through Key
     Pre  => Key'Length > 0 and Key'First = 1 and
             Count (Dic) < Capacity (Dic),
     Post => Has (Dic, X) and
     Item (Dic, Key) = X and
     Count (Dic) = Count (Dic)'Old + 1;

private
   Max_Dic : constant := 100;
   subtype Dic_Count is Natural range 0 .. Max_Dic;
   type Element is range 0.. 100;
   type Key_Value_Pair (Str_Len : Key_Index := 64) is record
         Key : Key_String (1 .. Str_Len);
         Value : Element;
      end record;
   type Dic_Array is array (1 .. Max_Dic) of Key_Value_Pair;
   type Dictionary is record
      Count : Dic_Count;
      Entries : Dic_Array;
   end record;
end Dictionaries;
