with System.Storage_Elements;
package body Device with
SPARK_Mode,
  Refined_State => (State =>
                      (Old_X, Status_Port, Register))
is
   Old_X : Integer := 0;

   Status_Port : Integer with
     Address => System.Storage_Elements.To_Address (16#FFFF_FFF0#),
     Volatile,
     Async_Writers;

   Register : Integer with
     Address => System.Storage_Elements.To_Address (16#FFFF_FFF4#),
     Volatile,
     Async_Readers,
     Effective_Writes;

   Read_Ack : constant := -1;
   subtype Wait_Time is Natural range 1 .. 20000;

     pragma Warnings (Off, """Register"" might not be written");
     procedure Write (X : Integer; Success : out Boolean) with
     Refined_Global => (Output => Register,
                        Input  => Status_Port,
                        In_Out => Old_X)
   is
      Status_Value : Integer;
   begin
      if X /= Old_X then
         Old_X := X;
         Register := X;
         for I in Wait_Time loop
            Status_Value := Status_Port;
            Success := Status_Value = Read_Ack;
            exit when Success;
         end loop;
      else
          Success := True;
      end if;
   end Write;

   function Last_Written return Integer is (Old_X) with
     Refined_Global => Old_X;

begin
   pragma SPARK_Mode (Off);
   -- presumed initialization of Register
end Device;
