with Ada.Containers.Vectors;
package Simple_Sets.Model with
SPARK_Mode
is
   type Model_Set (<>) is private;

   function Equivalent (S : Set; M : Model_Set) return Boolean;

   function To_Model (S : Set) return Model_Set with
     Post => Equivalent (S, To_Model'Result),
     Convention => Ghost;

   function Add_Member (M : Model_Set; E : Integer) return Model_Set with
      Convention => Ghost;


   function Preexisting_Members_Invariant (S : Set; M : Model_Set)
                                           return Boolean with
     Convention => Ghost;

   function Number_Of_Members (M : Model_Set) return Natural with
   Convention => Ghost;

   function Is_Member (M : Model_Set; E : Integer) return Boolean with
   Convention => Ghost;

   function Get_Member (M : Model_Set; N : Positive) return Integer with
     Pre => Number_Of_Members (M) /= 0,
     Convention => Ghost;

   function Equivalent (A, B : Model_Set) return Boolean is
      (Number_Of_Members (A) = Number_Of_Members (B) and then
       (for all I in 1 .. Number_Of_Members (A) =>
               Is_Member (B, Get_Member (A, I)))) with
     Convention => Ghost;

   function "=" (Left, Right : Model_Set) return Boolean renames Equivalent;

private
   pragma SPARK_Mode (Off);

   package Vector_Sets is new Ada.Containers.Vectors (Positive, Integer, "=");
   use Vector_Sets;
   use type Ada.Containers.Count_Type;

   type Model_Set is record
      V : Vector;
   end Record;

end Simple_Sets.Model;
