package Simple_Sets is
   -- The type is limited because it is implemented as an access type.
   -- Making the type limited avoids the problems associated with
   -- copying pointers.
   type Set is limited private;

   -- Given a set, S, returns the number of members in the set.
   function Number_Of_Members (S : Set) return Natural;

   -- Given an element, E, which is a ptotential member of a set, S,
   -- returns True if the element is a member of the set. False otherwise.
   function Is_Member (S : Set; E : Integer) return Boolean;

   -- Returns the N'th member of the set.  Allows iteration of the members
   -- of the set.  There is no implied order in the indexing and the N'th
   -- element may not be the same as the set is changed.
   -- A run-time exception might be raised if the set has no members.
   function Get_Member (S : Set; N : Positive) return Integer with
     Pre => N <= Number_Of_Members (S);

   -- Initialize a set before use.
   procedure Init (S : out Set) with
     Post => Number_Of_Members (S) = 0;

   -- Add a member to the set if it is not already present in the set.
   procedure Add_Member (S : in out Set; E : Integer);

   -- Destroy the set.  All storage used by the set is deallocated.
   procedure Destroy_Set (S : in out Set) with
     Post => Number_Of_Members (S) = 0;

   -- True if the two sets, A and B, have the same number of elements
   -- and the same members.
   function Equivalent (A, B : Set) return Boolean;

private
   type Set_Record;

   type Set is access Set_Record;

   type Set_Record is record
      Element : Integer;
      Next    : Set;
   end record;

end Simple_Sets;
