package A_Stack
with SPARK_Mode,
  Abstract_State => The_Stack,
  Initializes => The_Stack,
  Initial_Condition => Is_Empty and Count (The_Stack) = 0
is
   Stack_Size : constant := 100;
   type Stack_Type (<>) is private;
   subtype Item is Integer range 0 .. 20;

   function The_Stack return Stack_Type with
     Global => The_Stack,
     Convention => Ghost;

   function Count (S : Stack_Type) return Natural with
     Convention => Ghost;
   function Element (S : Stack_Type; N : Positive) return Item with
     Pre => N <= Count (S),
     Convention => Ghost;
   function Unchanged (Invariant_Part,
                       New_Stack : Stack_Type) return Boolean is
     (for all I in 1 .. Count (Invariant_Part) =>
           Element (Invariant_Part, I) = Element (New_Stack, I)) with
     Pre => Count (Invariant_Part) <= Count (New_Stack),
     Convention => Ghost;

   function Is_Empty return Boolean with
     Global => The_Stack,
     Post => Is_Empty'Result = (Count (The_Stack) = 0);
   function Is_Full return Boolean with
     Global => The_Stack,
     Post => Is_Full'Result = (Count (The_Stack) = Stack_Size);
   function Top return Item with
     Global => The_Stack,
     Pre => not Is_Empty or else Count (The_Stack) > 0;

   procedure Push (It : in Item) with
     Global => (In_Out => The_Stack),
     Pre => not Is_Full or else Count (The_Stack) < Stack_Size,
     Post => Top = It and
             Unchanged (The_Stack'Old, The_Stack) and
             Count (The_Stack) = Count (The_Stack)'Old + 1 and
             Element (The_Stack, Count (The_Stack)) = It;
   procedure Pop (It : out Item) with
     Global => (In_Out => The_Stack),
     Pre => not Is_Empty or else Count (The_Stack) > 0,
     Post => It = Top'Old and
             Unchanged (The_Stack, The_Stack'Old) and
             Count (The_Stack) = Count (The_Stack)'Old - 1;

private
   subtype Stack_Pointer is Natural range 0 .. Stack_Size;
   subtype Index is Stack_Pointer range 1 .. Stack_Size;
   type Vector is array (Index) of Item;
   type Stack_Type is record
      P : Stack_Pointer;
      V : Vector;
   end record;

end A_Stack;
