package Disc_Stacks
with SPARK_Mode
is
   subtype Item is Integer range 0 .. 20;
   subtype Allowable_Stacks is Natural range 0 .. 1000;
   type Stack (Stack_Size : Allowable_Stacks := 100) is private;

   function Count (S : Stack) return Natural with
     Convention => Ghost;
   function Unchanged (Invariant_Part, New_Stack : Stack) return Boolean with
     Pre => Count (Invariant_Part) <= Count (New_Stack),
     Convention => Ghost;

   function Is_Not_Empty (S : Stack) return Boolean with
     Post => Is_Not_Empty'Result = (Count (S) > 0);
   function Is_Full (S : Stack) return Boolean with
     Post => Is_Full'Result = (Count (S) = S.Stack_Size);
   function Top (S : Stack) return Item with
     Pre => Is_Not_Empty (S) or Count (S) > 0;

   procedure Clear (S :    out Stack) with
     Post => not Is_Not_Empty (S) and Count (S) = 0;

   procedure Push  (S : in out Stack; It : in     Item) with
     Pre  => not Is_Full (S) or Count (S) < S.Stack_Size,
     Post => Unchanged (S'Old, S) and
             Top (S) = It and
             Count (S) = Count (S)'Old + 1;

   procedure Pop   (S : in out Stack; It :    out Item) with
     Pre  => Is_Not_Empty (S) or Count (S) > 0,
     Post => It = Top (S)'Old and
             Unchanged (S, S'Old) and
             Count (S) = Count (S)'Old - 1;

private
   type Vector is array (Allowable_Stacks range <>) of Item;
   type Stack (Stack_Size : Allowable_Stacks := 100) is record
      P : Allowable_Stacks;
      V : Vector (1 .. Stack_Size);
   end record;
end Disc_Stacks;
