package Stacks with
  SPARK_Mode
is
   subtype Item is Integer range 0 .. 20;
   type Stack is private;
   Stack_Size : constant := 100;

   function Count (S : Stack) return Natural with
     Convention => Ghost;
   function Element (S : Stack; N : Positive) return Item with
     Pre => N <= Count (S),
     Convention => Ghost;
   function Unchanged (Invariant_Part,
                       New_Stack : Stack) return Boolean is
     (for all I in 1 .. Count (Invariant_Part) =>
           Element (Invariant_Part, I) = Element (New_Stack, I)) with
     Pre => Count (Invariant_Part) <= Count (New_Stack),
     Convention => Ghost;

   function Is_Empty (S : Stack) return Boolean with
     Post => Is_Empty'Result = (Count (S) = 0);
   function Is_Full (S : Stack) return Boolean with
     Post => Is_Full'Result = (Count (S) = Stack_Size);
   function Top (S : Stack) return Item with
     Pre => not Is_Empty (S) or Count (S) > 0;

   procedure Clear (S :    out Stack) with
     Post => Is_Empty (S) and Count (S) = 0;

   procedure Push  (S : in out Stack; It : in     Item) with
     Pre  => not Is_Full (S) or Count (S)  < Stack_Size,
     Post => Unchanged (S'Old, S) and
             Top (S) = It and
             Count (S) = Count (S)'Old + 1 and
             Element (S, Count (S)) = It;

   procedure Pop   (S : in out Stack; It :    out Item) with
     Pre  => not  Is_Empty (S) or Count (S) > 0,
     Post => It = Top (S)'Old and
             Unchanged (S, S'Old) and
             Count (S) = Count (S)'Old - 1;

private
   subtype Stack_Pointer is Natural range 0 .. Stack_Size;
   subtype Index is Stack_Pointer range 1 .. Stack_Size;
   type Vector is array (Index) of Item;
   type Stack is record
      P : Stack_Pointer;
      V : Vector;
   end record;
end Stacks;
